#pragma once
#include <vector>
#include <list>
#include <fstream>
#include <string>
#include <glm\gtc\matrix_transform.hpp>

enum Mtyp {
	null,
	dynamicMesh,
	cloth,
	bone,
	staticMesh,
	shadowMesh = 6
};

struct ChunkHeader {
	char name[5];
	std::uint32_t size;
	std::streampos position;
};

struct Modl {
	std::string name			= "";
	std::string parent			= "";
	Mtyp type					= null;
	std::int32_t renderFlags	= -1;
	glm::mat4 m4x4Translation	= glm::mat4(1.0f);
	std::string texture			= "";
	float* vertex				= nullptr;
	float* uv					= nullptr;
	std::uint32_t* mesh			= nullptr;
	std::uint32_t meshSize		= 0;
};


class Object
{
public:
	Object(const char* path);
	~Object();

private:

	std::vector<Modl*> vModls;
	std::fstream fsMesh;
	std::vector<std::string> vTextures;


private:
	void loadChunks(std::list<ChunkHeader*> &destination, std::streampos start, const std::uint32_t end);
	void analyseMsh2Chunks(std::list<ChunkHeader*> &chunkList);
	void analyseMatdChunks(std::list<ChunkHeader*> &chunkList);
	void analyseModlChunks(Modl* dataDestination, std::list<ChunkHeader*> &chunkList);
	void analyseGeomChunks(Modl* dataDestination, std::list<ChunkHeader*> &chunkList);
	void analyseSegmChunks(Modl* dataDestination, std::list<ChunkHeader*> &chunkList);
	void analyseClthChunks(Modl* dataDestination, std::list<ChunkHeader*> &chunkList);
	void readVertex(Modl* dataDestination, std::streampos position);
	void readUV(Modl* dataDestination, std::streampos position);
	

public:
	std::vector<Modl*> getModels() const;

};
